# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _

from jumbo.middleware.thread_local import get_current_user
from jumbo.utils.functions import get_settings
from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel

from organization.models import Organization

######################################
# PRESS REVIEWS                      #
######################################
DELIVERY = [
    ("d", _("Dayly"),),
    ("w", _("Weekly"),),
    ("f", _("Fortnightly"),),
    ("m", _("Monthly"),),
    ("t", _("Two-monthly"),),
    ("q", _("Quarterly"),),
    ("s", _("Six-monthly")),
    ("a", _("Annual"))
]

class PressReviewManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()
        qs = super(PressReviewManager, self).get_query_set()

        ## se può vedere tutte le press review ritorno il QS non filtrato
        if user.has_perm("press.list_all_press_review"):
            return qs

        if user.has_perm("press.list_organization_press_review"):
            return qs.filter(
                organizations__contacts__user=user,
                status=1
            )

        return qs

class PressReview(DateModel, UserModel, OrderedModel, StatusModel):
    organizations = models.ManyToManyField(
        Organization,
        db_index = True,
        related_name = "press_reviews",
        blank = True, null = True,
        verbose_name = _("Organizations"),
    )
    name = models.CharField(
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )
    slug = models.SlugField(
        max_length = 100,
        db_index = True,
        verbose_name = _('Slug'),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Note"),
    )
    ordering_list = models.CharField(
        max_length = 200,
        blank = True, null = True,
        help_text = _("""
            Inserire ordinamento lista campi delle press separati da virgole.
        """),
        verbose_name = _('Ordering list')
    )
    list_fields = models.CharField(
        max_length = 200,
        blank = True, null = True,
        help_text = _("""
            Inserire lista campi delle press separati da virgole che si vogliono
            far visualizzare al cliente.
        """),
        verbose_name = _('List fields')
    )
    delivery = models.CharField(
        max_length = 1,
        choices = DELIVERY,
        default = "m",
        verbose_name = _('Deadline')
    )
    objects = PressReviewManager()

    class Meta:
        verbose_name = _("Press Review")
        verbose_name_plural = _("Press Review")
        ordering = ['name',]
        permissions = (
            ("view_press_review",               "Can view press review"),
            ("list_all_press_review",           "Can list all press review"),
            ("list_organization_press_review",  "Can list organization press review"),
        )

    def __unicode__(self):
        return u"%s" % self.name

    def get_list_organizations(self):
        return u', '.join([org.name for org in self.organizations.all()])
    get_list_organizations.short_description = _('organizations')

######################################
# THEME                              #
######################################
class Theme(DateModel, UserModel, OrderedModel, StatusModel):
    name = models.CharField(
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )
    slug = models.SlugField(
        max_length = 100,
        db_index = True,
        verbose_name = _('Slug'),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Note"),
    )

    class Meta:
        verbose_name = _("Theme")
        verbose_name_plural = _("Themes")
        ordering = ['name',]
        permissions = (
            ("view_theme",             "Can view theme"),
        )

    def __unicode__(self):
        return u"%s" % self.name

######################################
# SECTOR                             #
######################################
class Sector(DateModel, UserModel, OrderedModel, StatusModel):
    name = models.CharField(
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )

    class Meta:
        verbose_name = _("Sector")
        verbose_name_plural = _("Sectors")
        ordering = ['name',]
        permissions = (
            ("view_sector",             "Can view sector"),
        )

    def __unicode__(self):
        return u"%s" % self.name

######################################
# FORMAT                             #
######################################
class Format(DateModel, UserModel, OrderedModel, StatusModel):
    name = models.CharField(
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )

    class Meta:
        verbose_name = _("Format")
        verbose_name_plural = _("Formats")
        ordering = ['name',]
        permissions = (
            ("view_format",             "Can view format"),
        )

    def __unicode__(self):
        return u"%s" % self.name

######################################
# SOURCE                             #
######################################
PERIODICITY = (
    ('P', _('Period')),
    ('D', _('Daily')),
)

class Source(DateModel, UserModel, OrderedModel, StatusModel):
    sector = models.ForeignKey(
        Sector,
        related_name = "sources",
        blank = True, null = True,
        db_index = True,
        verbose_name = _('Sector'),
    )
    formats = models.ManyToManyField(
        Format,
        related_name = "sources",
        blank = True, null = True,
        db_index = True,
        verbose_name = _('Formats'),
    )
    periodicity = models.CharField(
        max_length = 1,
        choices = PERIODICITY,
        db_index = True,
        default = 'D',
        verbose_name = _('Periodicity'),
    )
    name = models.CharField(
        unique = True,
        max_length = 100,
        db_index = True,
        verbose_name = _('Name'),
    )
    slug = models.SlugField(
        max_length = 100,
        db_index = True,
        verbose_name = _('Slug'),
    )
    description = models.TextField(
        blank = True, null = True,
        verbose_name = _("Note"),
    )

    class Meta:
        verbose_name = _("Source")
        verbose_name_plural = _("Sources")
        ordering = ['name',]
        permissions = (
            ("view_source",             "Can view source"),
        )

    def __unicode__(self):
        return u"%s" % self.name

    def get_list_formats(self):
        if self.formats:
            return u', '.join([format.name for format in self.formats.all()])
    list_formats = property(get_list_formats)

######################################
# PRESS                              #
######################################
def get_press_upload_to(instance, filename):
    return "presses/%s/%s/%s" % (instance.source, instance.title, filename)

class PressManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()
        qs = super(PressManager, self).get_query_set()

        ## se può vedere tutti i press ritorno il QS non filtrato
        if user.has_perm("press.list_all_presses"):
            return qs.distinct("id")

        if user.has_perm("press.list_organization_presses"):
            return qs.filter(
                press_reviews__organizations__contacts__user=user,
                status=1, press_reviews__status=1,
            ).distinct("id")

        return qs.distinct("id")

class Press(DateModel, UserModel, OrderedModel, StatusModel):
    themes = models.ManyToManyField(
        Theme,
        db_index = True,
        related_name = "presses",
        verbose_name = _("Themes"),
    )
    press_reviews = models.ManyToManyField(
        PressReview,
        db_index = True,
        related_name = "presses",
        verbose_name = _("Press Review"),
    )
    code = models.CharField(
        max_length = 300,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Title"),
    )
    date_p = models.DateField(
        db_index = True,
        verbose_name = _("Date")
    )
    delivery_date = models.DateField(
        db_index = True,
        null = True, blank = True,
        verbose_name = _("Delivery date")
    )
    title = models.CharField(
        max_length = 300,
        db_index = True,
        verbose_name = _("Title"),
    )
    slug = models.SlugField(
        max_length = 100,
        db_index = True,
        verbose_name = _('Slug'),
    )
    source = models.ForeignKey(
        Source,
        db_index = True,
        related_name = "presses",
        verbose_name = _("Source"),
    )
    file = models.FileField (
        upload_to = get_press_upload_to,
        verbose_name = _("File"),
    )
    abstract = models.TextField (
        null = True, blank = True,
        verbose_name = _("Abstract"),
    )
    keywords = models.CharField(
        null = True, blank = True,
        max_length = 200,
        db_index = True,
        verbose_name = _("Keywords"),
    )
    note = models.TextField (
        null = True, blank = True,
        verbose_name = _("Note"),
    )
    objects = PressManager()

    class Meta:
        verbose_name = _("Press")
        verbose_name_plural = _("Press")
#        ordering = ['themes__name', '-date_p', 'title']
        permissions = (
            ("view_press",                  "Can view press"),
            ("list_all_presses",            "Can list all press"),
            ("list_organization_presses",   "Can list organization press"),
            ("download_press",              "Can download press"),
            ("download_organization_press", "Can download organization press"),
            ("export_press_csv",            "Can export press in csv"),
        )

    def __unicode__(self):
        return u"%s" % self.title

    def save(self, force_insert=False, force_update=False):
        if not self.delivery_date:
            self.delivery_date = self.date_p
        super(Press, self).save(force_insert=force_insert, force_update=force_update)
        
    def get_list_themes(self):
        return u', '.join([theme.name for theme in self.themes.all()])
    get_list_themes.short_description = _('themes')
    get_list_themes.admin_order_field = 'themes__name'

    def get_list_pressreviews(self):
        return u', '.join([press_review.name for press_review in self.press_reviews.all()])
    get_list_pressreviews.short_description = _('press review')

    def can_be_downloaded(self):
        user = get_current_user()
        if user.is_superuser:
            return True
        if user.has_perm("press.download_press"):
            return True
        if user.has_perm("press.download_organization_press"):
            press_reviews_of_press = self.press_reviews.all()
            press_reviews_of_user = user.contact.organization.press_reviews.all()
            if press_reviews_of_user.filter(id__in=press_reviews_of_press):
                return True
        return False
